#pragma once

#include <windows.h>
#include <eh.h>
#include <stdio.h>

///////////////////////////

//@@	The base exception class

class	BaseException
{
public:

	BaseException( const char * const err )
	{
		strncpy( err_buf, err, 128 );
	}

	//@@	Returns the description of the error

	virtual const char * what()		{	return err_buf;	}

protected:

	char	err_buf[128];		//@@	store the error description
								//		should be a std::string instead but I use a char buf
								//		for simplicity
};

//////////////

//@@	Window GetLastError() exception class
//
//		Whenever u encounter an error from a win32 call, just throw an instance of this class

class	WindowException	:	public	BaseException
{
public:

	WindowException()	:	BaseException("")
	{
		// translate the last error code to a system string

		FormatMessage(	FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL, 
						GetLastError(),
						MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ), // Default language
						err_buf,
						128,
						NULL );
	}

};

///////////////

//@@	The hardware exception class

class	SEHException	:	BaseException
{
public:

	//@@	Translator function to translate SEH exceptions to C++ exceptions
	//
	//		Remember this must be set for each spawned thread.

	static	void SEH_Translator( unsigned int u, EXCEPTION_POINTERS *exp )
	{
		char	buf[128];			// should be a std::ostringstream but char buffer for simplicity

		switch( exp->ExceptionRecord->ExceptionCode )
		{
		case	EXCEPTION_ACCESS_VIOLATION:		_snprintf( buf, 128, "Access violation");	break;
		case	EXCEPTION_INT_DIVIDE_BY_ZERO:	_snprintf( buf, 128, "Divide by zero");		break;

			// fill in the rest yourself from EXCEPTION_POINTERS structure

		default:	_snprintf( buf, 128, "Unknown hardware exception");
		}

		throw SEHException( buf );
	}


	//@@	Constructor

	SEHException( const char * const err ) : BaseException( err )	{}

};

////////////////////////////